#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <stdlib.h>
#include <math.h>
#include "../src/good.h"
#include "../src/stockex.h"
#include "../src/queue.h"
#include "../src/auxiliaire.h"
#include "../src/global.h"
#include "../src/loop.h"

#ifndef __OPT__
#define __OPT__
static int seed = 4;
static int turn_number= 1000;
static int player_number= 20;
static int test_number = 100000;
////////////////////////////////////////////////////////////////
// Function for parsing the options of the program
// Currently available options are :
// -s <seed> : sets the seed
// -m <turn_number> : sets the turn number
// -n <player_number> : sets the player number
// -c <stockex_number> : sets the stockex number
// -b <test_number> : sets the number of test
void parse_opts2(int argc, char* argv[]) {
  int opt;
  while ((opt= getopt(argc,argv,"s:m:n:b:")) != -1) {
    switch (opt) {
    case 's':
      seed = atoi(optarg);
      break;
    case 'm':
      turn_number= atoi(optarg);
      break;
    case 'n':
      player_number= atoi(optarg);
      break;
    case 'b':
      test_number = atoi(optarg);
      break;
    default: /* '?' */
      fprintf(stderr, "Usage : %s [-s seed] [-n player_number] [-m turn_number] [-b test_number]\n",argv[0]);
      exit(EXIT_FAILURE);
      }
  }
  if (player_number > 20){
    player_number=20;
  }
  if(stockex_number > 100){
    stockex_number=100;
  }
}


////////////////////////////////////////////////////////////////
#endif

void printf_OK(char* test)
{
  printf("[\033[32m OK \033[0m] %s \n",test);
}

void printf_FAIL(char* test)
{
  printf("[\033[31m FAIL \033[0m] %s \n",test);
}

int winner_is_smart(struct queue* queue, int player_number)
{
  if (player_number<0) {
    printf("[ ERROR ] \033[031m player_number error \033[0m    \t ");
    return -1;
  }
  if (queue->length_queue==0 || queue==NULL){
    printf("[ ERROR ] \033[031m queue error \033[0m        \t \t ");
    return -1;
  }
  else{
    int tab_results[player_number];
    struct teddy* tab_teddy[player_number];
    for(int i=0;i<player_number;i=i+1){
      struct teddy* actual_teddy=queue__pop(queue);
      tab_results[i]=wallet__value(actual_teddy->initialisation);
      printf("Teddy %d : %d \n",i,tab_results[i]);
      tab_teddy[i]= actual_teddy;
    }
    int max=0;
    int position_max=-1;
    for(int i=0;i<player_number;i=i+1){
      if (max<=tab_results[i]){
	max=tab_results[i];
	position_max=i;
      }
    }
    struct teddy* winner=tab_teddy[position_max];
    return(winner->type);
  }
}

void starting_stockex3(struct stockex* Catane)
{
  size_t length_trans=rand()%MAX_TRANSACTION;
  if(length_trans==0){length_trans=1;}
  struct transac* transaction_tab[length_trans];
  for(size_t j = 0 ; j <length_trans ; j=j+1)
    {
      struct wallet a = random_wallet();
      struct wallet b = random_wallet();
      for(int k = 0; k < MAX_GOOD; k=k+1){
        if(a.data[k] > b.data[k]){
          a.data[k] = a.data[k] - b.data[k];
          b.data[k] = 0;
        }
        else{
          b.data[k] = b.data[k] - a.data[k];
          a.data[k] = 0;
        }
      }
      struct transac c; //création transaction évitant malloc
      c.wallet_in = a;
      c.wallet_out = b;
      c.actual_place = Catane;
      c.next_place = Catane;
      transaction_tab[j]=&c;
    }
  Catane->length=length_trans;
  Catane->mark=0;
  for (size_t i = 0; i < length_trans; i = i + 1) {
    Catane->transaction_tab[i] = *transaction_tab[i];
  }
}

const struct stockex* starting_stockex_bis()
{
  fill_name_stockex(stockex_number);
  created_stockex = malloc(stockex_number*(sizeof(struct stockex)+8));
  for(int i = 0; i < stockex_number; i ++){
    created_stockex[i].name = name_stockex[i];
    starting_stockex3(&created_stockex[i]);
  }
  link_transac(created_stockex, stockex_number);
  return created_stockex;
}


void test_ours_smart(int n)
{
  int error = 0;
  int no_play = 0;
  int smart_wins = 0;
  struct queue queue;
  for(int i = 0; i < n; i ++)
    {
      if (player_number==0 || turn_number==0 || stockex_number==0)
	no_play ++;
      else{
	const struct stockex* tab_stockex = starting_stockex_bis();
	queue = fill_queue(player_number, tab_stockex);
	int global_time = 0;
	while (global_time < turn_number){
	  struct teddy* active_teddy = queue__pop(&queue);
	  global_time = global_time+active_teddy->priority_teddy;
	  struct play active_teddy_play=which_transac(active_teddy, stockex_number);
	  active_teddy->initialisation=active_teddy_play.transac;
	  int time_spent = active_teddy_play.time;
	  queue__push(&queue, time_spent, active_teddy);
	}
      }
      int which_teddy = winner_is_smart(&queue, player_number);
      if(which_teddy == -1)
	error ++;
      if(which_teddy == 1)
	smart_wins ++;
      seed = seed + 1;
      srand(seed);
      printf("FIN %d \n",i);
    }
  printf("Sur %d tour de jeu: \n -Il y a %d de tour de jeu non joué \n -Il y a %d tour de jeu avec une erreur \n -L'ours intelligent gagne %d fois\n",n,no_play,error,smart_wins);
}


// //

//RED : \O33[31m
//GREEN : \033[32m


int main(int argc, char* argv[])
{
  parse_opts2(argc,argv);
  srand(seed);
  //stockex_number=rand()%MAX_STOCKEX;
  stockex_number=1;
    printf("[\033[33m TESTS \033[0m] sur l'ours smart  : \n");
    test_ours_smart(test_number);
  return 0;
}
